// Forlix SrcDS Manager
// http://forlix.org/, df@forlix.org
//
// Copyright (c) 2010-2013 Dominik Friedrichs

#include "SrcDS_Manager.h"


static MT_ServerData *FillServerDataStruct(CHashStr *ht)
{
  MT_ServerData *sd = new MT_ServerData;
  memset(sd, 0, sizeof(MT_ServerData));

  const char *tmp;

  try
  {
    if(tmp = ht->Read("FriendlyName"))
      strlcpy(sd->FriendlyName, sizeof(sd->FriendlyName), tmp);
    else
      throw("FriendlyName");


    if(tmp = ht->Read("Path"))
      strlcpy(sd->Path, sizeof(sd->Path), tmp);
    else
      throw("Path");

    if(tmp = ht->Read("GameDir"))
      strlcpy(sd->GameDir, sizeof(sd->GameDir), tmp);
    else
      throw("GameDir");


    if(tmp = ht->Read("Host"))
      strlcpy(sd->Host, sizeof(sd->Host), tmp);
    else
      throw("Host");

    if(tmp = ht->Read("Port"))
      strlcpy(sd->Port, sizeof(sd->Port), tmp);
    else
      throw("Port");


    if(tmp = ht->Read("MaxPlayers"))
      strlcpy(sd->MaxPlayers, sizeof(sd->MaxPlayers), tmp);
    else
      throw("MaxPlayers");

    if(tmp = ht->Read("MapName"))
      strlcpy(sd->MapName, sizeof(sd->MapName), tmp);
    else
      throw("MapName");


    if(tmp = ht->Read("MiscArgs"))
      strlcpy(sd->MiscArgs, sizeof(sd->MiscArgs), tmp);
    else
      throw("MiscArgs");


    if(tmp = ht->Read("Priority"))
      strlcpy(sd->Priority, sizeof(sd->Priority), tmp);
    else
      throw("Priority");

    if(tmp = ht->Read("Affinity"))
      strlcpy(sd->Affinity, sizeof(sd->Affinity), tmp);
    else
      throw("Affinity");


    if(tmp = ht->Read("MaxMemory"))
      strlcpy(sd->MaxMemory, sizeof(sd->MaxMemory), tmp);
    else
      throw("MaxMemory");

    if(tmp = ht->Read("MonitorDelay"))
      strlcpy(sd->MonitorDelay, sizeof(sd->MonitorDelay), tmp);
    else
      throw("MonitorDelay");


    if(tmp = ht->Read("UpdPath"))
      strlcpy(sd->UpdPath, sizeof(sd->UpdPath), tmp);
    else
      throw("UpdPath");

    if(tmp = ht->Read("UpdArgs"))
      strlcpy(sd->UpdArgs, sizeof(sd->UpdArgs), tmp);
    else
      throw("UpdArgs");
  }
  catch(const char *key)
  {
    LogMsg("Missing key \"%s\" for server \"%s\"", key, sd->FriendlyName);

    delete sd;
    return(NULL);
  }

  return(sd);
}


static bool ReadToken(char *dst, size_t len,
                      const char *&str)
{
  if(!dst || !len)
    return(false);

  unsigned int clen = 0;
  unsigned int tlen = 0;

  dst[0] = '\0';

  if(*str == '"')
  // treat as quoted token
  {
    char *eq = strchr(str+1, '"');

    if(!eq)
    // ending quote not found
      return(false);

    clen = eq-++str;
    tlen = clen+1;
  }
  else
  // treat as bare token
  {
    clen = strcspn(str, "\x20\t\"");

    if(!clen
    || str[clen] == '"')
    // zero length token
    // or ending quote (disallowed)
      return(false);

    tlen = clen;
  }

  if(clen >= len)
  // truncate to fit into dst
    clen = len-1;

  strncpy(dst, str, clen);
  dst[clen] = '\0';

  str += tlen;
  return(true);
}


static bool AlignToken(const char *&str,
                       bool requirews=false)
{
  const char *skip = str;

  while(*str == '\x20'
     || *str == '\t')
    str++;

  if(!*str
  || (requirews && str == skip))
  // no token aligned
  // or missing required whitespace
    return(false);

  return(true);
}


bool ReadConfig(MT_ServerData *mtsd[],
                size_t MaxServers)
{
  FILE *fcfg = fopen(CFG_PATH, "rt");

  if(!fcfg)
  // failed to open config file
  {
    LogMsg("Failed to read config file \"%s\"", CFG_PATH);
    return(false);
  }

  enum CfgState {Root, HaveSectionName, InSection};
  CfgState cstate = Root;

  CHashStr fht(64);
  CHashStr dpht(64);

  char ln[512];
  unsigned int m = 0;
  unsigned int line= 0;

  bool result = true;

  try
  {
    while(fgets(ln, sizeof(ln), fcfg) != NULL)
    {
      char *lp = &ln[0];
      line++;

      // skip leading whitespaces
      AlignToken(lp);

      // chop off \r\n
      chomp(lp);

      // skip empty and comment lines
      if(!*lp || strpequ(lp, "//"))
        continue;

      switch(cstate)
      {
        case Root:
        {
          char fname[64];

          if(!ReadToken(fname, sizeof(fname), lp)
          || !*fname
          || AlignToken(lp))
          // error reading token
          // or zero length server name
          // or trailed by another token
            throw("Failed to parse server name");

          if(dpht.Read(fname) != NULL)
            throw("Server name duplicate");

          fht.Write("FriendlyName", fname);
          dpht.Write(fname, "1");

          cstate = HaveSectionName;
        }
        break;

        case HaveSectionName:
        {
          if(*lp != '{')
          // not the beginning of a section
            throw("Expected '{'");

          if(AlignToken(++lp))
          // trailed by another token
            throw("Expected standalone '{'");

          cstate = InSection;
        }
        break;

        case InSection:
        {
          if(*lp == '}')
          // section end
          {
            if(AlignToken(++lp))
            // trailed by another token
              throw("Expected standalone '}'");

            if(m >= MaxServers)
              throw("MaxServers overflow");

            mtsd[m] = FillServerDataStruct(&fht);
            fht.Wipe();

            if(!mtsd[m++])
              throw("Failed to read out server keys");

            cstate = Root;
            break;
          }

          // split into key/value and write to hash table
          char key[64];
          char value[512];

          if(!ReadToken(key, sizeof(key), lp)
          || !AlignToken(lp, true))
          // error reading token
          // or key not trailed by another token
            throw("Failed to parse key");

          if(!ReadToken(value, sizeof(value), lp)
          || AlignToken(lp))
          // error reading token
          // or value trailed by another token
            throw("Failed to parse value");

          fht.Write(key, value);
        }
        break;
      }
    }
  }
  catch(const char *str)
  {
    LogMsg("%s at line %u", str, line);
    result = false;
  }

  if(result
  && cstate != Root)
  {
    LogMsg("Config file terminated unexpectedly at line %u", line);
    result = false;
  }

  fclose(fcfg);
  return(result);
}
