// Forlix SrcDS Manager
// http://forlix.org/, df@forlix.org
//
// Copyright (c) 2010-2013 Dominik Friedrichs

#include "SrcDS_Manager.h"

#include <string.h>
#include <time.h>
#include <stdio.h>
#include <stdarg.h>


// strlcpy - truncating version of strcpy

size_t strlcpy(char *dst,
               size_t size,
               const char *src)
{
  if(!dst || !src)
    return(0);

  char *d = dst;
  const char *s = src;
  size_t n = size;

  if(n && --n)
  {
    do
    {
      if(!(*d++ = *s++))
        break;
    }
    while(--n);
  }

  if(!n)
  {
    if(size)
      *d = '\0';
    while(*s++);
  }

  return(s-src-1);
}


// strlcat - truncating version of strcat

size_t strlcat(char *dst,
               const char *src,
               size_t size)
{
  if(!dst || !src)
    return(0);

  char *d = dst;
  const char *s = src;
  size_t n = size;
  size_t dlen;

  while(n-- && *d)
    d++;

  dlen = d - dst;
  n = size - dlen;

  if(!n)
    return(dlen+strlen(s));

  while(*s)
  {
    if(n != 1)
    {
      *d++ = *s;
      n--;
    }
    s++;
  }

  *d = '\0';

  return(dlen+(s-src));
}



// strnequ - partial string testing for equality
// quite a bit faster than VC6's strncmp

bool strnequ(const char *str1,
             const char *str2,
             size_t num)
{
  if(!str1 || !str2 || !num)
    return(false);

  size_t p = 0;

  while(*str1 && ++p < num)
    if(*str1++ != *str2++)
      return(false);

  if(*str1 == *str2)
    return(true);

  return(false);
}


// strpequ - partial string testing for equality
// returns true if str begins with ref

bool strpequ(const char *str,
             const char *ref)
{
  if(!str || !ref)
    return(false);

  while(*ref)
    if(*str++ != *ref++)
      return(false);

  return(true);
}


// strequ - string testing for equality

bool strequ(const char *str1,
            const char *str2,
            bool case_sens)
{
  if(!str1 || !str2)
    return(false);

  if(case_sens)
    return(strcmp(str1, str2) == 0);
  else
    return(stricmp(str1, str2) == 0);
}


// strsplt - gets a substring from a buffer
// up to the specified character sequence

char *strsplt(char *dst,
              size_t len,
              const char *seq,
              const char *&src)
{
  if(!dst)
    return(NULL);

  char *p;
  p = strstr(src, seq);

  if(p)
  {
    *p = '\0';
    strlcpy(dst, len, src);

    *p = seq[0];
    src = p+strlen(seq);

    return(dst);
  }
  else
  {
    dst[0] = '\0';
    return(NULL);
  }
}


// chomp - chop off any trailing
// \r\n, \r, or \n in a string, return
// true if something was removed

bool chomp(char *str)
{
  if(!str || !*str)
    return(false);

  short p = strlen(str);
  short q = --p;

  if(str[p] == '\n')
    str[p--] = '\0';

  if(p >= 0 && str[p] == '\r')
    str[p--] = '\0';

  if(q-p > 0)
    return(true);

  return(false);
}


// thread-safe logging

void LogMsg(const char *format, ...)
{
  if(WaitForSingleObject(g_LogMutex, 250) != WAIT_OBJECT_0)
    return;

  FILE *flog = fopen(LOG_PATH, "at");

  if(!flog)
  {
    ReleaseMutex(g_LogMutex);
    return;
  }

  time_t time_u = time(NULL);
  struct tm *lcltime = localtime(&time_u);

  fprintf(flog, "L %04u/%02u/%02u - %02u:%02u:%02u: ",
                lcltime->tm_year+1900, lcltime->tm_mon+1, lcltime->tm_mday,
                lcltime->tm_hour, lcltime->tm_min, lcltime->tm_sec);

  va_list args;

  va_start(args, format);
  vfprintf(flog, format, args);
  vfprintf(stdout, format, args);
  va_end(args);

  fprintf(flog, "\n");
  fclose(flog);

  ReleaseMutex(g_LogMutex);
  return;
}
