// Forlix SrcDS Manager
// http://forlix.org/, df@forlix.org
//
// Copyright (c) 2010-2013 Dominik Friedrichs

#include "SrcDS_Manager.h"


bool InstallService(const char *ServiceName,
                    const char *DisplayName,
                    const char *Description,
                    const char *LaunchCmd,
                    const char *Dependencies)
{
  SC_HANDLE hscmgr = OpenSCManager(NULL, NULL, SC_MANAGER_CREATE_SERVICE);

  if(!hscmgr)
  {
    printf("OpenSCManager failed with error %i\n", GetLastError());
    return(false);
  }

  SC_HANDLE hsvc = CreateService(hscmgr,
                                 ServiceName,
                                 DisplayName,
                                 SERVICE_ALL_ACCESS,
                                 SERVICE_WIN32_OWN_PROCESS,
                                 SERVICE_AUTO_START,
                                 SERVICE_ERROR_NORMAL,
                                 LaunchCmd,
                                 NULL,
                                 NULL,
                                 Dependencies,
                                 NULL,
                                 NULL);

  if(!hsvc)
  {
    printf("Failed to install service \"%s\" with error %i\n", ServiceName, GetLastError());
    CloseServiceHandle(hscmgr);
    return(false);
  }

  char desc[512];
  strncpy(desc, Description, sizeof(desc));
  desc[sizeof(desc)-1] = '\0';

  SERVICE_DESCRIPTION svcdesc;
  svcdesc.lpDescription = desc;
  ChangeServiceConfig2(hsvc, SERVICE_CONFIG_DESCRIPTION, &svcdesc);

  printf("Service \"%s\" successfully installed\n", ServiceName);

  CloseServiceHandle(hsvc);
  CloseServiceHandle(hscmgr);
  return(true);
}


bool UninstallService(const char *ServiceName)
{
  SC_HANDLE hscmgr = OpenSCManager(NULL, NULL, SC_MANAGER_CONNECT);

  if(!hscmgr)
  {
    printf("OpenSCManager failed with error %i\n", GetLastError());
    return(false);
  }

  SC_HANDLE hsvc = OpenService(hscmgr, ServiceName, SERVICE_ALL_ACCESS);

  if(!hsvc)
  {
    printf("OpenService failed with error %i\n", GetLastError());
    CloseServiceHandle(hscmgr);
    return(false);
  }

  bool result;
  SERVICE_STATUS tmpstatus;
  ControlService(hsvc, SERVICE_CONTROL_STOP, &tmpstatus);

  if(!DeleteService(hsvc))
  {
    printf("Failed to uninstall service \"%s\"\n", ServiceName);
    result = false;
  }
  else
  {
    printf("Service \"%s\" successfully uninstalled\n", ServiceName);
    result = true;
  }

  CloseServiceHandle(hsvc);
  CloseServiceHandle(hscmgr);
  return(result);
}
