// Forlix SrcDS Manager
// http://forlix.org/, df@forlix.org
//
// Copyright (c) 2010-2013 Dominik Friedrichs

#ifndef SRCDS_MANAGER_H
#define SRCDS_MANAGER_H

#define WIN32_LEAN_AND_MEAN
#define _WIN32_WINNT 0x500

#include <windows.h>
#include <conio.h>
#include <string.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <winsock2.h>

#include "CHashStr.h"
#include "CStrBuf.h"

#define APPVERSION            "Forlix SrcDS Manager v1.10"

#define SERVICE_NAME          "SrcDSMgr"
#define MAX_SERVERS           16
#define TICK_INTERVAL         1

#define TOOLUPDATE_TIMEOUT    60
#define MAINUPDATE_TIMEOUT    20*60

#define GAME_EXEC             "srcds.exe"
#define UPDATE_EXEC           "steamcmd.exe"

#define MRR_MARKERFILE        "update_request.txt"

#define CFG_PATH              "srcds_mgr.cfg"
#define LOG_PATH              "srcds_mgr.log"
#define UPDATELOG_PATH        "srcds_mgr_update.log"

#define WM_THREAD_TERMINATE   WM_APP+1

// GetProcessMemoryInfo without requiring the bulky Windows SDK
typedef struct
{
  DWORD cb;
  DWORD PageFaultCount;
  SIZE_T PeakWorkingSetSize;
  SIZE_T WorkingSetSize;
  SIZE_T QuotaPeakPagedPoolUsage;
  SIZE_T QuotaPagedPoolUsage;
  SIZE_T QuotaPeakNonPagedPoolUsage;
  SIZE_T QuotaNonPagedPoolUsage;
  SIZE_T PagefileUsage;
  SIZE_T PeakPagefileUsage;
}
PROCESS_MEMORY_COUNTERS;
typedef PROCESS_MEMORY_COUNTERS *PPROCESS_MEMORY_COUNTERS;

typedef BOOL (WINAPI *t_GetProcessMemoryInfo)(HANDLE Process,
                                              PPROCESS_MEMORY_COUNTERS ppsmemCounters,
                                              DWORD cb);

// ConfigFile.cpp
typedef struct
{
  char FriendlyName[64];

  char Path[MAX_PATH];
  char GameDir[32];

  char Host[64];
  char Port[8];

  char MaxPlayers[8];
  char MapName[64];

  char MiscArgs[128];

  char Priority[16];
  char Affinity[8];

  char MaxMemory[8];
  char MonitorDelay[8];

  char UpdPath[MAX_PATH];
  char UpdArgs[128];

  DWORD MonThreadId;
  HANDLE MonThreadHdl;
}
MT_ServerData;

bool ReadConfig(MT_ServerData *mtsd[],
                size_t MaxServers);

// Monitor.cpp
DWORD WINAPI MonitorThread(LPVOID lpParam);

// MonitorMisc.cpp
bool CreateServerProcess(const MT_ServerData *mtsd,
                         PROCESS_INFORMATION *pi);
bool CreateUpdateProcess(const MT_ServerData *mtsd,
                         PROCESS_INFORMATION *pi,
                         HANDLE &pipeout_read,
                         bool toolonly);

enum ServerHealth {Server_Healthy,
                   Server_MemoryFull,
                   Server_TimedOut,
                   Server_Terminated};

ServerHealth GetServerHealth(const MT_ServerData *mtsd,
                             PROCESS_INFORMATION *pi,
                             unsigned int *pf);

void FetchFromPipe(HANDLE pipe, CStrBuf *buf);

void WriteUpdateLog(const MT_ServerData *mtsd,
                    const char *update_buf);

// PingSrcDS.cpp
int PingSrcDS(const char *str_host,
              const char *str_port,
              unsigned int timeout);

// Misc.cpp
size_t strlcpy(char *dst,
               size_t size,
               const char *src);
bool strequ(const char *str1,
            const char *str2,
            bool case_sens=true);
bool strpequ(const char *str,
             const char *ref);
char *strsplt(char *dst,
              size_t len,
              const char *seq,
              const char *&src);
bool chomp(char *str);
void LogMsg(const char *format, ...);

// WinMisc.cpp
bool HasProcessTerminated(HANDLE hProcess);
bool FileExists(const char *path);
bool FolderExists(const char *path);
bool GetFileDirectory(char *directory,
                      const char *path);

// ServiceInstall.cpp
bool InstallService(const char *ServiceName,
                    const char *DisplayName,
                    const char *Description,
                    const char *LaunchCmd,
                    const char *Dependencies);
bool UninstallService(const char *ServiceName);

// Global vars
extern HANDLE g_LogMutex;
extern HANDLE g_UpdMutex;

extern t_GetProcessMemoryInfo GetProcessMemoryInfo;

#endif // SRCDS_MANAGER_H
